@if (@CodeSection == @Batch) @then


@echo off
setlocal EnableDelayedExpansion

rem VIBORAS.bat: A multi-player version in color of Snake game
rem Designed and developed by Antonio Perez Ayala aka Aacini
rem Dedicated to Salvador Medina (a very special Snake game player)

rem Use PowerShell engine to read control keys and show text in color as described at:
rem http://www.dostips.com/forum/viewtopic.php?f=3&t=6936&p=46206#p46206

rem Version 1.0 - 2018/06/05
rem     Fully working preliminary version. These planned features are not yet implemented:
rem     - Turbo food
rem     - Reverse food
rem     - Gate/Wall food
rem     - Coral food
rem     - Lapse food
rem     - Snake reproduction
rem     - "exit" variable

rem The full game documentation is in file VIBORAS.html


rem Check if parameter is a game definition .txt file or chaining label
for %%f in (%1) do (
   if /I "%%~Xf" equ ".txt" (
      if not exist "%~1" (
         echo Parameters definition file not found: "%~1"
         exit /B
      )
      set "gameParams=%~1"
   ) else (
      goto %1
   )
)

mode con cols=80 lines=40
chcp 850 > NUL
color 0F
echo/
echo/
for %%a in ("                           "
            "                                     "
            "                                     "
            "                                     "
            "                                "
            "                                         "
            "                                         "
            "                                         "
            "                                    B A T") do echo   %%~a
echo/
echo/
echo VIBORAS.bat game written by Antonio Perez Ayala aka Aacini
echo/
echo/
echo Use Left-Arrow and Right-Arrow keys to control snake turns
echo When the snake eats "the food", it increases its size and speed
echo/
echo/
echo Up to four snakes can be controlled via these keys:
echo/
echo #1 Yellow Snake:  Up-Arrow,   Left-Arrow, Down-Arrow, Right-Arrow
echo/
echo #2 Green Snake:   "W" letter, "A" letter, "S" letter, "D" letter
echo/
echo #3 Magenta Snake: "G" letter, "V" letter, "B" letter, "N" letter
echo/
echo #4 Cyan Snake:    "O" letter, "K" letter, "L" letter, ";" character
echo/
echo/ 
echo Press space bar to pause or quit the game
echo/
echo/
set /P "=Press any key to start the game . . . " < NUL
pause > NUL

cls
echo/
echo              
echo                     
echo                    
echo                
echo                             
cd /D "%~DP0" > pipeFile.txt
"%~F0" Input >> pipeFile.txt  |  "%~F0" Main < pipeFile.txt  |  "%~F0" Output
echo/
echo Ending game...
ping localhost -n 2 > NUL
del pipeFile.txt
goto :EOF



:Input
set "letter=ABCDEFGHIJKLMNOPQRSTUVWXYZ"
for /L %%i in (0,1,25) do (
   set /A "Letter!letter:~%%i,1!=%%i+65"
)
set "NextToL=188"  &  rem ... in English keyboards = semicolon. In Spanish keyboards should be 192 = "" letter.
set /A "Esc=27, Space=32, PageUp=33, Home=36, LeftArrow=37, UpArrow=38, RightArrow=39, DownArrow=40"
if not defined gameParams (
   set "snakes=1"
) else (
   for /F "delims=" %%a in ('findstr /I "snakes=" "%gameParams%"') do %%a
)

set "keys= %LeftArrow%='%LeftArrow%,S=1'; %RightArrow%='%RightArrow%,S=1'; %UpArrow%='%UpArrow%,S=1'; %DownArrow%='%DownArrow%,S=1';"
if %snakes% geq 2 (
   set "keys=%keys% %LetterA%='%LeftArrow%,S=2'; %LetterD%='%RightArrow%,S=2'; %LetterW%='%UpArrow%,S=2'; %LetterS%='%DownArrow%,S=2';"
)
if %snakes% geq 3 (
   set "keys=%keys% %LetterV%='%LeftArrow%,S=3'; %LetterN%='%RightArrow%,S=3'; %LetterG%='%UpArrow%,S=3'; %LetterB%='%DownArrow%,S=3';"
)
if %snakes% geq 4 (
   set "keys=%keys% %LetterK%='%LeftArrow%,S=4'; %NextToL%='%RightArrow%,S=4'; %LetterO%='%UpArrow%,S=4'; %LetterL%='%DownArrow%,S=4';"
)

PowerShell  ^
   $command = @{ %Space%='%Space%'; %keys% };  ^
   Start-Sleep -m 2000;  ^
   Write-Output 0;  ^
   $console = $Host.UI.RawUI;  ^
   while ( $console.KeyAvailable ) {  ^
      $key = $console.ReadKey('NoEcho,IncludeKeyUp').VirtualKeyCode;  ^
   }  ^
   $key = 0;  ^
   while ( $key -ne %LetterN% ) {  ^
      while ( $key -ne %Space%  -and  $key -ne %Esc% ) {  ^
         $key = $console.ReadKey('NoEcho,IncludeKeyUp').VirtualKeyCode;  ^
         Write-Output $command[$key];  ^
      }  ^
      if ( $key -eq %Space% ) {  ^
         $console.WindowTitle = 'PAUSED  -  Continue? [Y,N]';  ^
      } else {  ^
         $console.WindowTitle = 'PLAY AGAIN? [Y,N]';  ^
      }  ^
      while ( $key -ne %LetterY%  -and  $key -ne %LetterN% ) {  ^
         $key = $console.ReadKey('NoEcho,IncludeKeyUp').VirtualKeyCode;  ^
      }  ^
      Write-Output $key;  ^
      $console.WindowTitle = 'VIBORAS.bat by Aacini';  ^
   }
%End PowerShell%
exit /B


:Output
rem Parameters received via lines read have one of these forms:
rem - 'color',Color = Set default screen color
rem - 'beep',freq,millisecs = Play beep tone
rem - Pos = Move cursor to Position=Y<<7+X
rem - Pos,Color,'String' = Move cursor to Position and show the String in given Color
PowerShell  ^
   $console = $Host.UI.RawUI;  ^
   $console.WindowTitle = 'VIBORAS.bat by Aacini';  ^
   $curSize = $console.CursorSize;  ^
   $console.CursorSize = 0;  ^
   $coords = $console.CursorPosition;  ^
   foreach ( $line in $input ) {  ^
      $Pos,$Color,$String = $line.Split(',');  ^
      if ( $Pos -eq 'color' ) {  ^
         cmd /C color $Color  ^
      } elseif ( $Pos -eq 'beep' ) {  ^
         [Console]::Beep(([int]$Color),([int]$String))  ^
      } else {  ^
         $coords.Y = ([int]$Pos) -shr 7;  ^
         $coords.X = ([int]$Pos) %% 128;  ^
         $console.CursorPosition = $coords;  ^
         if ( $String ) {  ^
            $back = [int]('0x'+$Color[0]);  ^
            $fore = [int]('0x'+$Color[1]);  ^
            Write-Host  -BackgroundColor $back  -ForegroundColor $fore  -NoNewLine  $String  ^
         }  ^
      }  ^
   }  ^
   $console.CursorSize = $curSize;
%End PowerShell%
exit /B



:Main

rem Empty environment
(
   for /F "delims==" %%v in ('set') do set "%%v="
   set "path=%path%"
   set "SystemRoot=%SystemRoot%"
   set "gameParams=%gameParams%"
)

rem Define a basic game: standard size 80x40, single player, one standard food, no reserved foods
set /A cols=80, lines=40, snakes=1, stdFoods=1, resFoods=0

if defined gameParams (
   rem Read game parameters from given file
   for /F "usebackq delims=" %%a in ("%gameParams%") do %%a
   set "gameParams="
)
for /L %%S in (1,1,%snakes%) do if not defined name[%%S] set "name[%%S]=SNAKE #%%S"


REM Define control movement variables

set /A "PageUp=33, Home=36, LeftArrow=37, UpArrow=38, RightArrow=39, DownArrow=40"
set /A "ToRight=1, ToLeft=-1, ToDown=128, ToUp=-128"
set "turn[%ToRight%+%RightArrow%]=%ToDown%"  &  set "turn[%ToRight%+%LeftArrow%]=%ToUp%"
set "turn[%ToDown%+%RightArrow%]=%ToLeft%"   &  set "turn[%ToUp%+%LeftArrow%]=%ToLeft%"
set "turn[%ToLeft%+%RightArrow%]=%ToUp%"     &  set "turn[%ToLeft%+%LeftArrow%]=%ToDown%"
set "turn[%ToUp%+%RightArrow%]=%ToRight%"    &  set "turn[%ToDown%+%LeftArrow%]=%ToRight%"


REM Define other configuration parameters you may modify

rem Initial, maximum and stop snake speeds (as inverse delays), initial snake size
set /A "maxDelay=50, minDelay=8, stopDel=9999,  initSize=5"
rem Formula for increase speed (decrease delay) each time a food is eaten
set "delayDec=(delay[%%S]*19)/20"
rem Snake colors (yellow, green, magenta, cyan and background) and body characters
set "color[1]=E6" & set "color[2]=A2" & set "color[3]=D5" & set "color[4]=B3" & set "color[0]=0F"
set "char[%ToRight%]=" & set "char[%ToLeft%]=" & set "char[%ToDown%]=" & set "char[%ToUp%]="
set "char[%ToRight%+%RightArrow%]="  &  set "char[%ToUp%+%LeftArrow%]="
set "char[%ToDown%+%RightArrow%]="   &  set "char[%ToRight%+%LeftArrow%]="
set "char[%ToLeft%+%RightArrow%]="   &  set "char[%ToDown%+%LeftArrow%]="
set "char[%ToUp%+%RightArrow%]="     &  set "char[%ToLeft%+%LeftArrow%]="
rem Beep frequencies for food eaten and snake crash
set /A "beep[1]=660, beep[2]=524, beep[3]=392, beep[4]=330, beep[0]=262"


REM Initialize "board" variable with all board positions, from 0 up to 8063,
REM with characters that indicate cell contents: <space> = free cell, "X" letter = frame

set "board= "
for /L %%i in (1,1,12) do set "board=!board!!board!"
set "board=!board:~0,4032!!board:~0,4032!"
set /A "colsM1=cols-1, linesM1=lines-1, aux1=(linesM1<<7)+colsM1"
set "board=!board:~0,%aux1%!"
rem Frame, top and bottom
for /L %%x in (0,1,%colsM1%) do (
   set /A "top=%%x, topP1=top+1,  bot=(linesM1<<7)+%%x, botP1=bot+1"
   for /F "tokens=1-4" %%i in ("!top! !topP1! !bot! !botP1!") do (
      set "board=!board:~0,%%i!X!board:~%%j!"
      set "board=!board:~0,%%k!X!board:~%%l!"
   )
)
rem Frame, left and right
for /L %%y in (0,1,%linesM1%) do (
   set /A "lef=%%y<<7, lefP1=lef+1,  rig=(%%y<<7)+colsM1, rigP1=rig+1"
   for /F "tokens=1-4" %%i in ("!lef! !lefP1! !rig! !rigP1!") do (
      set "board=!board:~0,%%i!X!board:~%%j!"
      set "board=!board:~0,%%k!X!board:~%%l!"
   )
)
for %%v in (colsM1 linesM1 top topP1 bot botP1 lef lefP1 rig rigP1) do set "%%v="


REM Initialize "free" variable with up to 1632 positions comprised:  ** ** **
REM of 4 digits + one space each that form a sample taken from the:  ** ** **
REM total positions distributed in 9 "islands", as shown here:       ** ** **

rem Get number of colsFree:linesFree with the same cols:lines original relation
set /A "maxFree=1632, colsM2=cols-2, linesM2=lines-2, N=maxFree*linesM2/colsM2, Next=N/2, linesFree=N/Next"
for /L %%# in (1,1,6) do if !Next! gtr !linesFree! set /A "Next=(Next+linesFree)/2, linesFree=N/Next"
set /A "colsFree=(maxFree-1)/linesFree+1"
rem Get linesFree omitting 2 groups of lines at 1/3 and 2/3 from the total lines; see scheme above
set /A "m=linesM2-linesFree, l1=linesM2/3-m/4-1, l2=l1+m/2-1, l4=linesM2*2/3+m/4+3, l3=l4-(m-m/2)+1"
set "linesFree= "
for /L %%y in (1,1,%linesM2%) do set "linesFree=!linesFree!%%y "
if %m% geq 2 (
   set "linesFree=!linesFree: 1 = !"
   set "linesFree=!linesFree: %linesM2% = !"
   set /A l2-=1, l3+=1, m-=2
)
if %m% geq 2 (
   for /L %%y in (%l1%,1,%l2%) do set "linesFree=!linesFree: %%y = !"
   for /L %%y in (%l3%,1,%l4%) do set "linesFree=!linesFree: %%y = !"
)
rem Get colsFree omitting 2 groups of columns at 1/3 and 2/3 from the total columns; see scheme above
set /A "m=colsM2-colsFree, l1=colsM2/3-m/4-1, l2=l1+m/2-1, l4=colsM2*2/3+m/4+3, l3=l4-(m-m/2)+1"
set "colsFree= "
for /L %%x in (1,1,%colsM2%) do set "colsFree=!colsFree!%%x "
if %m% geq 2 (
   set "colsFree=!colsFree: 1 = !"
   set "colsFree=!colsFree: %colsM2% = !"
   set /A l2-=1, l3+=1, m-=2
)
if %m% geq 2 (
   for /L %%x in (%l1%,1,%l2%) do set "colsFree=!colsFree: %%x = !"
   for /L %%x in (%l3%,1,%l4%) do set "colsFree=!colsFree: %%x = !"
)
rem Create up to maxFree free positions from linesFree:colsFree
set "free= "
set "freeLen=0"
for %%y in (%linesFree%) do (
   for %%x in (%colsFree%) do if !freeLen! lss %maxFree% (
      set /A "m=(%%y<<7)+%%x, freeLen+=1"
      set "m=   !m!"
      set "free=!free!!m:~-4! "
   )
)
rem Remove free positions from a 5x5 square placed at center
set /A "maxFree=freeLen, freeLen-=25, center=(lines/2<<7)+cols/2, l1=center+2*ToUp+2*ToLeft, l3=l1+4*ToDown"
for /L %%y in (%l1%,%ToDown%,%l3%) do (
   set /A "l4=%%y+4*ToRight"
   for /L %%x in (%%y,%ToRight%,!l4!) do (
      set "m=   %%x"
      for %%m in ("!m:~-4!") do set "free=!free:%%~m =!"
   )
)
for %%v in (N Next linesFree colsFree m l1 l2 l3 l4) do set "%%v="


REM General use variables

rem Status area variables
if %snakes% equ 1 (
   set /A "aux2=0, aux1=cols/2-6"
) else (
   set /A "aux2=cols/snakes, aux1=cols-aux2"
)
for /L %%S in (1,1,%snakes%) do (
   set /A "status[%%S]=(lines+1<<7)+aux1+2,aux1=(aux1+aux2)%%cols"
)
rem Food to grow: Standard and Boost
set "grow=123456789"
rem Food to store: Turbo, Reverse and Gate/Wall
set "store=TRGW"
rem Turn and answer keys
set "turn=/%LeftArrow%/%RightArrow%/"
set /A "Space=32, LetterN=78, LetterY=89,  aux1=lines+6"


set "input="
:WaitPS for PowerShell Input part start signal
   set /P "input="
if not defined input goto WaitPS


if not defined init set "init=1"
for /L %%# in () do (

   if defined init (
      setlocal EnableDelayedExpansion
      set /A "%init%"
      set "init="

      %/* Show the initial board */%

      %/* cls command not work here, use mode instead */%
      mode con cols=%cols% lines=%aux1%
      %/* color command not work here, execute it in PS */%
      echo color,%color[0]%
      set "aux1="
      for /L %%i in (1,1,%colsM2%) do set "aux1=!aux1! "
      < NUL (
         set /P "=!aux1: =!"
         for /L %%i in (1,1,%linesM2%) do set /P "=!aux1!"
         set /P "=!aux1: =!"
      ) > CON
      for /L %%S in (1,1,%snakes%) do (
         echo !status[%%S]!,%color[0]:~0,1%!color[%%S]:~0,1!,!name[%%S]!
         set /A "aux1=status[%%S]+ToDown, aux2=aux1+ToDown"
         echo !aux1!,%color[0]:~0,1%!color[%%S]:~0,1!,Size: %initSize%
         echo !aux2!,%color[0]:~0,1%!color[%%S]:~0,1!,Delay: %maxDelay%
      )
      echo %center%,%color[0]%,

      %/* Get the initial Standard-Foods from "free" and insert they in "board" */%

      for /L %%n in (1,1,%stdFoods%) do (
         set /A "aux1=(!random!%%freeLen)*5+1, aux2=aux1+5"
         for /F "tokens=1,2" %%i in ("!aux1! !aux2!") do (
            set /A "aux1=!free:~%%i,4!, aux2=aux1+1, freeLen-=1"
            set "free=!free:~0,%%i!!free:~%%j!"
         )
         echo !aux1!,%color[0]%,1
         for /F "tokens=1,2" %%i in ("!aux1! !aux2!") do (
            set "board=!board:~0,%%i!1!board:~%%j!"
         )
      )

      %/* Get the initial Reserved-Foods from "free" and insert they in "board" */%

      for /L %%n in (1,1,%resFoods%) do (
         for /L %%S in (1,1,%snakes%) do (
            set /A "aux1=(!random!%%freeLen)*5+1, aux2=aux1+5"
            for /F "tokens=1,2" %%i in ("!aux1! !aux2!") do (
               set /A "aux1=!free:~%%i,4!, aux2=aux1+1, freeLen-=1"
               set "free=!free:~0,%%i!!free:~%%j!"
            )
            echo !aux1!,%color[0]:~0,1%!color[%%S]:~0,1!,1
            for /F "tokens=1,2" %%i in ("!aux1! !aux2!") do (
               set "board=!board:~0,%%i!1!board:~%%j!"
            )
            set "resFood[!aux1!]=%%S"
         )
      )

      %/* Initialize the first snakes */%

      for /L %%S in (1,1,%snakes%) do (
         set /A "headPos[%%S]=tailPos[%%S]=center, size[%%S]=tailGrow[%%S]=initSize, del[%%S]=delay[%%S]=maxDelay, beg[%%S]=1!time:~-4,1!!time:~-2!"
      )
      set /A "headMov[1]=tailMov[1]=toRight"
      if %snakes% geq 2 set /A "headMov[2]=tailMov[2]=toLeft"
      if %snakes% geq 3 set /A "headMov[3]=tailMov[3]=toDown"
      if %snakes% geq 4 set /A "headMov[4]=tailMov[4]=toUp"

      set /A "minSize=maxSize=initSize, seconds=0, lastSec=!time:~7,1!"


   )  %/* End init */%


   if "!lastSec!" neq "!time:~7,1!" set /A "seconds+=1, lastSec=!time:~7,1!"


   %/* Control module. Get and process input keys, or advance one position each del centiseconds */%

   set "S="
   set "input="
   set /P "input="
   if defined input (
      set /A "key=!input!"
      if "!key!" equ "%Space%" (
         call :Pause
         for /L %%S in (1,1,%snakes%) do set "beg[%%S]=1!time:~-4,1!!time:~-2!"
      ) else (
         for /F %%S in ("!S!") do for /F "tokens=1,2" %%j in ("!headMov[%%S]! !key!") do (

            if "!turn:/%%k/=!" neq "%turn%" (
               %/* LeftArrow or RightArrow */%
               echo !headPos[%%S]!,!color[%%S]!,!char[%%j+%%k]!
               set "headMov[%%S]=!turn[%%j+%%k]!"
               set "node[!headPos[%%S]!]=!headMov[%%S]!"
               set "beg[%%S]=1!time:~-4,1!!time:~-2!"

            ) else if !key! equ %UpArrow% (
               %/* Process Slow/Normal/Turbo speeds /%
               set /A "aux1=delay[%%S]*2"
               if !del[%%S]! equ %stopDel% (
                  set "mode=Slow "
                  set "del[%%S]=!aux1!"
               ) else if !del[%%S]! equ !aux1! (
                  set "mode=     "
                  set "del[%%S]=!delay[%%S]!"
               ) else if !del[%%S]! equ !delay[%%S]! (
                  if !T[%%S]! gtr 0 (
                     set "mode=Turbo"
                     set /A "del[%%S]=delay[%%S]/2, T[%%S]-=1"
                  )
               )

            ) else if !key! equ %DownArrow% (
               %/* Process Normal/Slow/Stop speeds and Reverse direction */%
               set /A "aux1=delay[%%S]*2, aux2=delay[%%S]/2"
               if !del[%%S]! equ !aux2! (
                  set "mode=     "
                  set "del[%%S]=!delay[%%S]!"
               ) else if !del[%%S]! equ !delay[%%S]! (
                  set "mode=Slow "
                  set "del[%%S]=!aux1!"
               ) else if !del[%%S]! equ !aux1! (
                  set "mode=Stop "
                  set "del[%%S]=%stopDel%"
               ) else if !R[%%S]! gtr 0 (
                  rem TO DO: Change direction to reverse
                  set "mode=     "
                  set /A "R[%%S]-=1"
               )

            ) else if !key! equ %Home% (
               rem TO DO: Process insert Gate

            ) else if !key! equ %PageUp% (
               rem TO DO: Process lay Egg

            )

            if defined mode (
               set /A "aux1=status[%%S]+2*ToDown, aux2=aux1+ToDown"
               echo !aux1!,%color[0]:~0,1%!color[%%S]:~0,1!,Delay: !del[%%S]!  
               if "!mode!" equ "     " (
                  echo !aux2!,%color[0]%,!mode!
               ) else (
                  echo !aux2!,!color[%%S]:~0,1!%color[0]:~0,1%,!mode!
               )
               set "mode="
            )

         )
      )
   ) else (
      %/* No input key, test if delay had passed */%
      for /L %%S in (1,1,%snakes%) do if not defined S (
         set /A "end[%%S]=1!time:~-4,1!!time:~-2!, elap=end[%%S]-beg[%%S], elap-=(elap>>31)*1000"
         if !elap! geq !del[%%S]! set /A "beg[%%S]=1!time:~-4,1!!time:~-2!, S=%%S"
      )
   )


   %/* If S variable is defined, specify the Snake to move */%

   if defined S (for /F %%S in ("!S!") do (

      %/* Update tail, excepting when snake grows */%

      if !tailGrow[%%S]! gtr 0 (set /A tailGrow[%%S]-=1) else (
         %/* Delete last tail. Next command have a space at end */%
         echo !tailPos[%%S]!,%color[0]%, 
         set /A "aux1=tailPos[%%S]+1"
         for /F "tokens=1,2" %%i in ("!tailPos[%%S]! !aux1!") do (
            set "board=!board:~0,%%i! !board:~%%j!"
         )
         %/* Insert last tail in "free", if needed */%
         if !freeLen! lss %maxFree% (
            set "aux1=   !tailPos[%%S]!"
            set "free=!free!!aux1:~-4! "
            set /A freeLen+=1
         )
         %/* Advance tail */%
         set /A "tailPos[%%S]+=tailMov[%%S]"
         if defined node[!tailPos[%%S]!] (
            set /A "tailMov[%%S]=node[!tailPos[%%S]!]"
            set "node[!tailPos[%%S]!]="
         )
      )

      %/* Advance head and process the new position */%

      set /A "headPos[%%S]+=headMov[%%S]"
      for /F %%i in ("!headPos[%%S]!") do for /F "delims=" %%c in ("!board:~%%i,1!") do (
         set "headChar=%%c"
         if "!grow:%%c=!" neq "%grow%" (
            %/* Food to eat and grow /*%
            if defined resFood[%%i] (
               %/* This is a reserved food */%
               if "!resFood[%%i]:~-1!" equ "%%S" set "headChar= "   %/* ... for this snake, OK */%
            ) else (
               set "headChar= "
            )
            if "!headChar!" equ " " (
               %/* Eat the food and update status */%
               set /A "size[%%S]+=%%c,tailGrow[%%S]+=%%c, aux1=status[%%S]+ToDown, aux2=aux1+ToDown, aux3=aux2+ToDown"
               if !size[%%S]! gtr !maxSize! set "maxSize=!size[%%S]!"
               if !delay[%%S]! gtr %minDelay% set /A "delay[%%S]=%delayDec%"
               set /A "del[%%S]=delay[%%S]"
               echo beep,!beep[%%S]!,300
               echo !aux1!,%color[0]:~0,1%!color[%%S]:~0,1!,Size: !size[%%S]!   
               echo !aux2!,%color[0]:~0,1%!color[%%S]:~0,1!,Delay: !del[%%S]! 
               echo !aux3!,%color[0]%,     
               %/* Get the next food from "free" */%
               set /A "aux1=(!random!%%freeLen)*5+1, aux2=aux1+5"
               for /F "tokens=1,2" %%i in ("!aux1! !aux2!") do (
                  set /A "aux1=!free:~%%i,4!, aux2=aux1+1, freeLen-=1"
                  set "free=!free:~0,%%i!!free:~%%j!"
               )
               set "aux3=1"
               if defined resFood[%%i] (
                  %/* The eaten food was reserved */%
                  if "!resFood[%%i]:~0,1!" equ "%%S" (
                     %/* The new food must also be reserved for this snake */%
                     if defined ResBoost (
                        set /A "aux3=%ResBoost%"
                        if !aux3! lss 1 (set "aux3=1") else if !aux3! gtr 9 set "aux3=9"
                     )
                     echo !aux1!,%color[0]:~0,1%!color[%%S]:~0,1!,!aux3!
                     set "resFood[!aux1!]=%%S"
                  ) else (
                     %/* The eaten food was BoostRes, cancel the new food */%
                     set "aux3=0"
                  )
                  set "resFood[%%i]="
               ) else (
                  if defined StdBoost (
                     set /A "aux3=%StdBoost%"
                     if !aux3! lss 1 (set "aux3=1") else if !aux3! gtr 9 set "aux3=9"
                  )
                  echo !aux1!,%color[0]%,!aux3!
               )
               %/* Insert the new food in "board", if not was cancelled */%
               if !aux3! neq 0 for /F "tokens=1,2" %%i in ("!aux1! !aux2!") do (
                  set "board=!board:~0,%%i!!aux3!!board:~%%j!"
               )
            )
            %/* Reset all timers */%
            for /L %%S in (1,1,%snakes%) do set /A "beg[%%S]=1!time:~-4,1!!time:~-2!"
         ) else if "!store:%%c=!" neq "%store%" (
            %/* Food to store */%
            rem TO DO: Store the food in this snake: Turbo, Reverse, or Gate/Wall
            set "headChar= "
         ) else if "%%c" equ "C" (
            %/* Coral food */%
            rem TO DO: Set values to convert this snake in a Coral one
            set "headChar= "
         ) else if "%%c" equ "L" (
            %/* Lapse food */%
            rem TO DO: Set values to enable Lapse mode for this snake
         )
      )

      %/* If new head position is valid */%

      if "!headChar!" equ " " (
         set /A "aux1=headPos[%%S]+1"
         set "aux2=   !headPos[%%S]!"
         for /F "tokens=1-4 delims=," %%i in ("!headPos[%%S]!,!headMov[%%S]!,!aux1!,!aux2:~-4!") do (
            %/* Show head at new position */%
            echo %%i,!color[%%S]!,!char[%%j]!
            %/* Update "board" variable */%
            set "board=!board:~0,%%i!#!board:~%%k!"
            %/* If new head position is in "free" variable, remove it */%
            set "aux1=!free: %%l = !"
            if "!aux1!" neq "!free!" (
               set "free=!aux1!"
               set /A freeLen-=1
            )
         )
      ) else (
         %/* Snake crash */%
         call :EndGame !headPos[%%S]! !headChar! !color[%%S]!
         %/* Start a new game */%
         endlocal
      )

   )) else if defined BoostRes (

      %/* Process creation of Boost-Reserved food */%

      set /A "aux3=%BoostRes%"
      if !aux3! gtr 0 (
         if !aux3! gtr 9 set "aux3=9"
         %/* Create a Boost-Reserved food for all smallest snakes */%
         for /L %%S in (1,1,%snakes%) do if !size[%%S]! equ !minSize! (
            %/* Get the next food from "free" and insert it in "board" */%
            set /A "aux1=(!random!%%freeLen)*5+1, aux2=aux1+5"
            for /F "tokens=1,2" %%i in ("!aux1! !aux2!") do (
               set /A "aux1=!free:~%%i,4!, aux2=aux1+1, freeLen-=1"
               set "free=!free:~0,%%i!!free:~%%j!"
            )
            echo !aux1!,%color[0]:~0,1%!color[%%S]:~0,1!,!aux3!
            set "resFood[!aux1!]=R%%S"
            for /F "tokens=1,2" %%i in ("!aux1! !aux2!") do (
               set "board=!board:~0,%%i!!aux3!!board:~%%j!"
            )
         )
      )

   ) else if defined Turbo (

      rem TO DO: Process creation of Turbo food

   ) else if defined Reverse (

      rem TO DO: Process creation of Reverse food

   ) else if defined Gate (

      rem TO DO: Process creation of Gate/Wall food

   ) else if defined Coral (

      rem TO DO: Process creation of Coral food

   ) else if defined Lapse (

      rem TO DO: Process creation of Lapse food

   )

)
exit never reached


:Pause
   set "key="
   set /P "key="
   if "%key%" equ "%LetterY%" exit /B
   if "%key%" equ "%LetterN%" (
      echo %status[1]%
      exit
   )
goto Pause


:endGame Position Char Color
CScript //nologo //E:JScript "%~F0" "{ESC}"
set "key="
:newGame
   echo %1,%color[0]%,%2
   echo beep,%beep[0]%,600
   echo %1,%3, 
   call :Delay 80
   set /P "key="
   if "%key%" equ "%LetterY%" exit /B
   if "%key%" equ "%LetterN%" (
      echo %status[1]%
      exit
   )
goto newGame


:Delay centisecs
set "b=1%time:~-2%"
:wait
   set /A "e=1%time:~-2%, elap=e-b, elap-=(elap>>31)*100"
if %elap% lss %1 goto wait
exit /B


@end


// JScript section used to send a keystroke to :Input PowerShell section
WScript.CreateObject("WScript.Shell").SendKeys(WScript.Arguments(0))

